/*
	This is our ongoing example of a collection (or container) class.
	
	We will be examining the inner workings of collection classes
	over the next couple of weeks.
	
	In the future, instead of having to write your own version of
	a collection class, you can use one of the pre-existing collection
	classes (e.g., ArrayList) that are part of the Java API.
	
*/

public class CDCollection {

	public static final int DEFAULT_COLLECTION_SIZE = 100;

	//instance variables
	private CD[] cdArray; // the array
	private int cdCount;  // logical size


	// constructors

	public CDCollection (int capacity) {
		cdArray = new CD[capacity];
		cdCount = 0;
	}

	public CDCollection () {
		this(DEFAULT_COLLECTION_SIZE);
	}

	public class CollectionBoundsException extends Exception {
	
		public CollectionBoundsException () {
			super();
		}
		public CollectionBoundsException (String info) {
			super(info);
		}
	}


	// simple accessors

	public CD get (int index) throws CollectionBoundsException {
		if (index < 0 || index >= cdCount) // error
			throw new CollectionBoundsException (index + " is an illegal index into the collection");
		return cdArray[index];
	}
	public int getSize() {
		return cdCount;
	}

	public String toString() {
	// Returns a string representing the entire contents of the CDCollection
		String result = "    CDCollection[" + "used=" + cdCount + "/length=" + cdArray.length + "\n";
		for (int i = 0; i < cdCount; i++)
		    result += "      cdArray["+i+"]= " + cdArray[i] + "\n";
		return result + " ]";
    }
    
    
	// simple mutator
	public void append (CD anotherCD) {
		cdArray[cdCount] = anotherCD;
		cdCount++;
	}
	
		
	// SEARCHING (accessors)

	// a search method that searches for a CD title

	public int searchForTitle(String theTitle) {	
	// returns the location of the CD with the specified title
	// returns -1 if no CD matches
	
		int look = 0;
		while (look < cdCount) {
			if ( theTitle.equals (cdArray[look].getTitle()))
				return look;
			look++;
		}
		return -1; // because the title wasn't found
	}


	// FILTERING (accessors)


	// a filter that produces all CDs in the collection by an artist

	public CDCollection getAllByArtist(String theArtist) {
	// This method returns a CDCollection object consisting of all
	// CDs in this collection that are by the specified artist
	
		CDCollection newColl = new CDCollection(cdCount);
		
		for (int i = 0; i < cdCount; i++) {
			
			if (cdArray[i].getArtist().equals(theArtist))
				newColl.append(cdArray[i]);
		}
		return newColl;
	}

}